// Datei Hexer01.c
/* ------------------------------------------------------------------------
Gibt ein paar Adressen zur Standardausgabe aus (zur Orientierung des
Benutzers). Liest dann eine Ganzzahl von der Standardeingabe ein und
versucht, diese Zahl als Adresse zu interpretieren und den Inhalt der
entsprechenden Speicherstelle (8 Bytes) auszugeben. Wenn der Benutzer
eine "ungueltige Adresse" eingibt, wird das Programm vermutlich wegen
eines Zugriff-Fehlers (access violation) abgebrochen).
------------------------------------------------------------------------ */
#include <string.h> // fuer strlen
#include <stdio.h>  // fuer printf, sprintf, ...
#include <signal.h> // Signale ("Ausnahmen") senden und fangen
#include <stdlib.h> // atof, atoi, rand, malloc, calloc, free, exit,
                    // bsearch, qsort, abs, div, ...
// ------------------------------------------------------------------------
// Die Adressen der folgenden Variablen werden ausgegeben:
int   v1    = -1;
char  c1[8] = {'A', 'B', 'C', '1', '2', '3', 'a', 'b'};
int   i1    = -1;
int  *p1    = &i1;
// ------------------------------------------------------------------------
// Die folgende Variable ist nicht-lokal, damit sie in der Funktion main
// benutzt und von der Funktion beHandler "korrigiert werden" kann:
char * adressec = (char *) c1; // Vom Benutzer eingegebene Adresse
// ------------------------------------------------------------------------
void hilfeAusgeben(void) {
   // Auch die Adressen der folgenden Variablen werden ausgegeben:
   int   v2    = -1;
   char  c2[8] = {'A', 'B', 'C', '1', '2', '3', 'a', 'b'};
   int   i2    = -1;
   int  *p2    = &i2;

   printf("----------------------------------------\n");
   printf("Hexer01, Benutzung:\n");
   printf("Beenden:                 'q'\n");
   printf("Diese Hilfe ausgeben:    'h'\n");
   printf("8 Bytes vorwaerts:       'v' oder Return\n");
   printf("8 Bytes zurueck:         'z'\n");
   printf("----------------------------\n");
   printf("Oder eine Adresse eingeben,\n");
   printf("z.B. eine der folgenden:\n");
   printf("&v1: %#8x\n", &v1);
   printf(" c1: %#8x\n",  c1);
   printf(" p1: %#8x\n",  p1);
   printf("&v2: %#8x\n", &v2);
   printf(" c2: %#8x\n",  c2);
   printf(" p2: %#8x\n",  p2);
   printf("----------------------------------------\n");
} // hilfeAusgeben
// ------------------------------------------------------------------------
void beHandler(int sigNr) {
   printf("\nEin Signal Nr %d wurde empfangen!\n", sigNr);
   printf("Das Programm Hexer01 muss deshalb beendet werden!\n");
   exit(1);
} // beHandler
// ------------------------------------------------------------------------
int main(void) {

   int i;                  // Indexvariable fuer for-Schleifen

   char          ein[20];  // Die Eingabe des Benutzers als string
   unsigned char char1;    // Ein Byte des Speichers als unsigned char
   unsigned char lhb;      // Linkes  Halbbyte (fuer Hex-Darstellung)
   unsigned char rhb;      // Rechtes Halbbyte (fuer Hex-Darstellung)

   // Tabelle mit hexaedzimalen Ziffern (zum Uebestzen von lhb und rhb):
   char hextab[16] = {'0', '1', '2', '3', '4', '5', '6', '7',
                      '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
   const int LEN   = 8;    // Soviele Bytes werden ausgegeben:
   // ---------------------------------------------------------------------
   // Diverse Initialisierungen:
   signal(SIGSEGV, beHandler);
   hilfeAusgeben();

   // Wiederholt ein Kommando oder eine Adresse einlesen und ausfuehren.
   // Eine Adresse entspricht dem Kommando, ab dieser Speicherstelle
   // 8 Bytes auf dem Bildschirm anzuzeigen:
   while (1) {
      printf("Eine Adresse (oder 'q', 'h', 'v', 'z' oder nur Return)? ");
      fgets(ein, 20, stdin);           // Einen String nach ein lesen

      // Wenn der Benutzer nur auf die Return-Taste gedrueckt hat,
      // enthaelt ein nur 1 Zeichen (naemlich '\0'):
      if (strlen(ein) == 1 || ein[0] == 'v') {
         adressec += LEN;              // LEN Bytes vorwaerts
      } else if (ein[0] == 'z') {
         adressec -= LEN;              // LEN Bytes rueckwaerts
      } else if (ein[0] == 'h') {
         hilfeAusgeben();              // Bedienungsanleitung ausgeben
         continue;
      } else if (ein[0] == 'q') {
         break;                        // Programm beenden
      } else {
         sscanf(ein, "%x", &adressec); // Eingabe in Adresse umwandeln
      } // if
      // ------------------------------------------------------------------
      // LEN viele Bytes hexadezimal ausgeben:
      printf("%#x: ", adressec);
      for (i=0; i<LEN; i++) {
         lhb = *(adressec+i);  // Kopie des Bytes fuer linkes  Halbbyte
         rhb = *(adressec+i);  // Kopie des Bytes fuer rechtes Halbbyte
         lhb = lhb >> 4;       // Die linken 4 Bits nach rechts schieben
         rhb = rhb & 0x0F;     // Die linken 4 Bits auf 0 setzen
         lhb = hextab[lhb];    // Linkes  Halbbyte in Zeichen uebersetzen
         rhb = hextab[rhb];    // Rechtes Halbbyte in Zeichen uebersetzen
         printf("%c%c ", lhb, rhb); // Beide Halbbytes ausgeben
      }
      printf("\n");
      // ------------------------------------------------------------------
      // LEN viele Bytes als Zeichen ausgeben. Nicht-druckbare Zeichen
      // werden durch einen Punkt '.' ersetzt:
      printf("%#x: ", adressec);
      for (i=0; i<LEN; i++) {
         char1 = *(adressec+i);
         // Nicht-druckbare Zeichen durch Punkt '.' ersetzen:
         if (char1 <=32 || 254 <= char1) char1 = '.';
         printf(" %c ", char1);
      }
      printf("\n");
   } // while
      // ------------------------------------------------------------------
   printf("Hexer01: Das wars erstmal!\n");

   return 0;
} // main
/* ------------------------------------------------------------------------
Ein Dialog mit dem Programm HexAnzeiger:

----------------------------------------
Hexer01, Benutzung:
Beenden:                 'q'
Diese Hilfe ausgeben:    'h'
8 Bytes vorwaerts:       'v' oder Return
8 Bytes zurueck:         'z'
----------------------------
Oder eine Adresse eingeben,
z.B. eine der folgenden:
&v1: 0x403010
 c1: 0x403014
 p1: 0x40301c
&v2: 0x22fe54
 c2: 0x22fe48
 p2: 0x22fe44
----------------------------------------
Eine Adresse (oder 'q', 'h', 'v', 'z' oder nur Return)? 403010
0x403010: FF FF FF FF 41 42 43 31
0x403010:  .  .  .  .  A  B  C  1
Eine Adresse (oder 'q', 'h', 'v', 'z' oder nur Return)? v
0x403018: 32 33 61 62 FF FF FF FF
0x403018:  2  3  a  b  .  .  .  .
Eine Adresse (oder 'q', 'h', 'v', 'z' oder nur Return)? z
0x403010: FF FF FF FF 41 42 43 31
0x403010:  .  .  .  .  A  B  C  1
Eine Adresse (oder 'q', 'h', 'v', 'z' oder nur Return)? q
Hexer01: Das wars erstmal!
------------------------------------------------------------------------ */
