// Datei Hexer02.c
/* ------------------------------------------------------------------------
Liest von der Standardeingabe den Pfadnamen einer Datei D ein, liest die
Datei D und gibt ihren Inhalt in hexadezimaler Darstellung und in zeichen-
weiser Darstellung zur Standardausgabe aus.

Der Benutzer kann durch Eingabe der folgenden Kommandos navigieren:
su (eine Seite nach unten), so (eine Seite nach oben), zu (eine Zeile nach
unten), zo (eine Zeile nach oben), uu (zum Ende der Datei), oo (zum Anfang
der Datei). Wenn er nur auf Return drueckt, wird das letzte Kommando
wiederholt. Wenn er 'q' eingibt, wird das Programm beendet.

Eine volle Seite besteht aus 17 Zeilen. In einer vollen Zeile werden 16
Zeichen der Datei D dargestellt, zuerst (links) hexadezimal und dann
(rechts) als Zeichen. Man denke sich die Zeichen der Datei D mit 0 be-
ginnend durchnummeriert. Am Anfang einer Zeile auf dem Bildschirm steht die
Nummer des Zeichens, welches als erstes in dieser Zeile dargestellt wird.
Bei der zeichenweisen Darstellung (rechts) werden Kontrollzeichen (iscntrl)
und transparente Zeichen (isspace) durch Punkte '.' ersetzt.

Das Kommando "Seite nach unten" (su) bewirkt, dass in der Datei
D um 16 Zeilen (nicht um 17 Zeilen) vorgerueckt wird. Entsprechend wird
beim Kommando "Seite nach oben" nur um 16 Zeilen (nicht um 17) zurueckge-
gangen. Somit "ueberlappen sich" die Seiten jeweils um eine Zeile.

In den letzten drei Zeilen des Bildschirms steht eine Kurzanleitung fuer
das Programm (Erlaeuterung der Kommandos zum Navigieren).
------------------------------------------------------------------------ */
#include <ctype.h>   // fuer iscntrl, isspace
#include <string.h>  // fuer strcpy, strlen
#include <windows.h> // fuer SetConsoleCursorPosition, COORD, HANDLE, ...
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...

// #define TEST
   #undef  TEST
// ------------------------------------------------------------------------
// Konstanten fuer das Layout:
int const ZEICHEN_PRO_ZEILE =  16;
int const  ZEILEN_PRO_SEITE =  17;
int const ZEICHEN_PRO_SEITE = 272; // 16 * 17
int const ZEILE_PFADNAME    =   0; // Zeile zum Fragen nach dem Pfadnamen
int const ZEILE_LAENGE      =   1; // Zeile fuer Ausgabe der Dateilaenge
int const ZEILE_D1          =   3; // Erste  "richtige Datenzeile"
int const ZEILE_D17         =  19; // Letzte "richtige Datenzeile"
int const ZEILE_KOMMANDO    =  20; // Fuer Ausgabe des Benutzer-Kommandos
int const ZEILE_BED_ANLEIT  =  22; // Fuer Bedienungs-Anleitung

HANDLE hStdout;
HANDLE hStdin;
// ------------------------------------------------------------------------
void gotoXY(int x, int y) {
   // Funktioniert nur unter Windows95/98/NT/ME/2000 etc.
   // Bringt den Bildschirmzeiger (cursor) in die Zeile x, Spalte y
   // (Zeilen und Spalten sind mit 0 beginnend durchnumeriert):
   COORD c;

   c.X = x;
   c.Y = y;
   SetConsoleCursorPosition(hStdout, c);
} // gotoXY
// ------------------------------------------------------------------------
int minInt(int n1, int n2) {
   if (n1 < n2) return n1;
   return n2;
} // minInt
// ------------------------------------------------------------------------
void gibBedienungsAnleitungAus(void) {
   // Gibt eine kurze (3 Zeilen) Bedienungs-Anleitung zum Bildschirm aus:

   gotoXY(0, ZEILE_BED_ANLEIT);
   printf(
   "Kommandos: zu, zo (Zeile nach unten/oben), nur Return (letzen Befehl nochmal)\n"
   "           su, so (Seite nach unten/oben), q (quit)\n"
   "           uu, oo (zum Ende/Anfang der Datei)"
   );
   fflush(stdout);

} // GibBedienungsAnleitungAus
// ------------------------------------------------------------------------
void gibBlanksAus(unsigned int anz) {
   for (; anz > 0; anz--) {
      printf(" ");
   }
   fflush(stdout);
} // gibBlanksAus
// ------------------------------------------------------------------------
void gibZeileAus(char * s, int const abHier) {
   // Gibt eine Zeile aus, auf der maximal ZEICHEN_PRO_ZEILE viele Zeichen
   // von s ab der Stelle s[abHier] dargestellt werden. Im linken Teil der
   // Zeile werden die Zeichen hexadezimal, im rechten Teil zeichenweise
   // ("als char-Werte") dargestellt. Bei der zeichenweisen Darstellung
   // werden Kontrollzeichen und transparente Zeichen durch Punkte ersetzt.

   int i; // Index fuer for-Schleifen

   // Tabelle zum Umwandeln von Zahlen zwischen 0 und 15 in hex-Ziffern:
   static char const tab[16] = {'0', '1', '2', '3', '4', '5', '6', '7',
                                '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
   char hexL, hexR, c; // Linke/rechte hex-Ziffer, ein ganzes Zeichen c

   // Wieviele Zeichen sollen auf dieser Zeile dargestellt werden (falls
   // die Zeile nicht voll ist)?
   int soViele = minInt((int)(strlen(s)-abHier), ZEICHEN_PRO_ZEILE);

   // Am Zeilenanfang die Nummer (den Index) des ersten Zeichens ausgeben:
   printf("%6d: ", abHier);
   fflush(stdout);
// cout << setw(6) << abHier << ":";

   // soViele Zeichen hexadezimal darstellen (linker Teil der Zeile):
   for (i=abHier; i<abHier+soViele; i++) {
      hexL = hexR = s[i];
      hexL >>= 4;
      hexL  &= 0x0F;
      hexR  &= 0x0F;
      printf("%c%c ", tab[hexL], tab[hexR]);  // 2 hex-Ziffern ausgeben
   }
   gibBlanksAus((ZEICHEN_PRO_ZEILE-soViele)*3);

   // soViele Zeichen zeichenweise darstellen (rechter Teil der Zeile):
   for (i=abHier; i<abHier+soViele; i++) {
      c = s[i];
      if (iscntrl(c) || isspace(c)) c = '.'; // Durch Punkte ersetzen
      printf("%c", c);
//    cout << c;
   }
   gibBlanksAus(ZEICHEN_PRO_ZEILE-soViele);
   printf("\n");
   fflush(stdout);

// cout << FUELLUNG_RECHTS << endl;          // Falls Zeile nicht-voll

} // gibZeileAus
// ------------------------------------------------------------------------
void gibSeiteAus(char * s, int abHier) {
   // Gibt ZEILEN_PRO_SEITE viele Zeilen (d.h. ein Seite) zum Bildschirm
   // aus. "Richtige Zeilen" dienen dazu, Zeichen von s (ab der Stelle
   // s[abHier] darzustellen. Falls s zu kurz oder abHier zu gross ist,
   // werden entsprechend viele Blankzeilen ausgegeben, um den alten Inhalt
   // dieser Zeilen auf dem Bildschirm zu loeschen.

   int zeilenNr;             // Index fuer for-Schleifen

   int nochZeichen         ; // Wieviele Zeichen sind auszugeben?
   int nochRichtigeZeilen  ;
   int nochBlankZeilen     ;

   nochZeichen         = strlen(s) - abHier; // Wieviele Zeichen sind auszugeben?
// nochZeichen         = s.size()  - abHier; // Wieviele Zeichen sind auszugeben?
   nochRichtigeZeilen  = (nochZeichen-1)/ZEICHEN_PRO_ZEILE + 1;
   nochRichtigeZeilen  = minInt(nochRichtigeZeilen, ZEILEN_PRO_SEITE);
   nochBlankZeilen     = ZEILEN_PRO_SEITE - nochRichtigeZeilen;

   #ifdef TEST
   gotoXY(0, 21);
   printf("nochRichtigeZeile: %d, nochBlankZeilen: %d, nochZeichen: %d ",
           nochRichtigeZeilen,    nochBlankZeilen,     nochZeichen);
   fflush(stdout);
   #endif // TEST

   // "Richtige Zeilen" ausgeben:
   gotoXY(0, ZEILE_D1);
   for (zeilenNr=0; zeilenNr<nochRichtigeZeilen; zeilenNr++) {
      gibZeileAus(s, abHier);
      abHier += ZEICHEN_PRO_ZEILE;
   }

   // Blankzeilen ausgeben:
   for (zeilenNr=0; zeilenNr<nochBlankZeilen; zeilenNr++) {
      gibBlanksAus(79);
      printf("\n");
      fflush(stdout);
   }
} // gibSeiteAus
// ------------------------------------------------------------------------
// Konstante Reihungen fuer das Unterprogramm fuehreKommandoAus
// (AK wie "Anzahl Kommandos"):
#define AK 6

// Fuer jede Kommando-Taste einen Ausgabe-Text:
const char * KOM_TEXT[AK] = {"Seite nach oben ",  "Seite nach unten",
                             "Datei-Anfang    ",  "Datei-Ende      ",
                             "Zeile nach oben ",  "Zeile nach unten"};

// Fuer jede Kommando-Taste der mit ihr eingegebene string:
const char * KOMMANDO[AK] = {"so",                "su",
                             "oo",                "uu",
                             "zo",                "zu"};

// Fuer jede Kommando-Taste: Um wieviel muss abHier veraendert werden?
// (Datei-Anfang und Datei-Ende sind Sonderfaelle):

// Die folgende Vereinbarung einer Reihung funktioniert leider nicht,
// weil -ZEICHEN_SEITE und -ZEICHEN_PRO_ZEILE keine konstanten
// Ausdruecke sind:
/*
   int K_DELTA   [AK] = {
      -ZEICHEN_PRO_SEITE, +ZEICHEN_PRO_SEITE,
                       0,                  0,
      -ZEICHEN_PRO_ZEILE, +ZEICHEN_PRO_ZEILE
   };
*/

// Haessliche Definition, die funktioniert:
const int K_DELTA[AK] = {-272, +272, 0, 0, -16, +16};

// ZEICHEN_PRO_SEITE und ZEICHEN_PRO_ZEILE mit #define zu definieren
// verursacht an anderen Stellen Fehlermeldungen.
// ------------------------------------------------------------------------
void fuehreKommandoAus(char * kommando, int * abHier, int datSize) {
   // Fuehrt ein Kommando des Benutzers aus. Beendet evtl. das Programm
   // (wenn der Benutzer 'q' eingegeben hat). Sonst wird der Index abHier
   // entsprechend dem Kommando veraendert. Der Parameter datSize sollte
   // die Laenge der auszugebenden Datei enthalten.

   int    index;  // fuer die Reihungen KOM_TEXT, KOMMANDO und K_DELTA
   char * kom_text = (char *) KOM_TEXT[1]; // Text  fuer Kommando "Seite runter"
   int    delta    =          K_DELTA [1]; // delta fuer Kommando "Seite runter"

   // Programm beenden? Auch "quit" oder "Qualle" sind erlaubt:
   if (kommando[0] == 'q' || kommando[0] == 'Q') {
      gotoXY(0, ZEILE_KOMMANDO);
      printf("Hexer02 beendet sich! ");
      fflush(stdout);
      exit(0);
   }

   // Das kommando analysieren und entsprechende Werte nach index, kom_text
   // und delta bringen. Unbekannte Eingaben werden als das Kommando
   // "Seite runter" interpretiert:

   for (index=0; index<AK; index++) {
      if (strcmp(kommando, KOMMANDO[index]) == 0) {
         kom_text  =  (char *) KOM_TEXT[index];
         delta     =  K_DELTA   [index];
         break;
      }
   }

   // Den Index abHier veraendern:
   *abHier += delta;

   // Die Sonderfaelle "zum Datei-Anfang" und "zum Datei-Ende" behandeln:
   if (index  == 2)       *abHier = 0;                           // Anfang
   if (index  == 3)       *abHier = datSize - ZEICHEN_PRO_SEITE; // Ende

   // Den Index abHier korrigieren, falls sein Wert "unvernuenftig" ist:
   if (*abHier >= datSize) *abHier = datSize - ZEICHEN_PRO_SEITE;
   if (*abHier < 0)        *abHier = 0;

   // Kommando-Text ausgeben (damit der Benutzer sieht, was passiert):
   gotoXY(0, ZEILE_KOMMANDO);
   printf("%s", kom_text);
   fflush(stdout);

} // fuehreKommandoAus
// ------------------------------------------------------------------------
int main(void) {

   int    abHier = 0;          // Ab eingabe[abHier] Zeichen ausgeben
   char aktuellesKommando[10]; //    letztes Kommando des Benutzers
   char   vorigesKommando[10]; // vorletztes Kommando des Benutzers
   int  i;                     // Index fuer for-Schleifen
   int  ezi;                   // Eingabezeichen als int

   FILE * fp;                  // Die anzuzeigende Datei

   char         einDat[100];   // Pfadname der Eingabedatei
   int          laenge;        // Laenge der eingelesenen Datei

   char         puffer [1000 * 1000]; // max. 1 MB
   // ---------------------------------------------------------------------
   hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
   hStdin  = GetStdHandle(STD_INPUT_HANDLE);

   // Bildschirm (25 Zeilen, 80 Spalten) loeschen:
   gotoXY(0, 0);
   gibBlanksAus(25 * 80);

   // Einen Pfadnamen einlesen bzw. einen Standardnamen nehmen:
   gotoXY(0,  ZEILE_PFADNAME);
   printf("Hexer02: Pfadname einer Datei? ");
   fflush(stdout);  // Damit der Cursor hinter der letzten Meldung steht!

   fgets(einDat, 100, stdin);        // Einen String nach einDat lesen
   einDat[strlen(einDat)-1] = '\0';  // Newline-Zeichen am Ende entfernen
   if (strlen(einDat) == 0) {
      strcpy(einDat, "Hexer02.c");   // Standard-Pfadname
   }

   // Versuch, eine Datei namens einDat zum Lesen (r) und als
   // Binaerdatei (b) zu oeffnen. Bei einer Textdatei (ohne das b)
   // werden unter Dos/Windows beim Einlesen die beiden Zeilenendezeichen
   // CR LF (hex: 0D 0A) durch ein LF-Zeichen ersetzt (so dass pro Zeile
   // ein Zeichen fehlt):
   fp = fopen(einDat, "rb");

   // Falls das Oeffnen der Datei nicht geklappt hat:
   if (fp == 0) {
      printf("Konnte Eingabedatei %s nicht oeffnen!\n", einDat);
      exit(-1);
   }

   gibBedienungsAnleitungAus();

   // Die Datei in die char-Reihung puffer lesen (maximal 1 MB):
   i = 0;
   while ((ezi = getc(fp)) != EOF) puffer[i++] = (char) ezi;
   laenge = i;
   puffer[i] = '\0';

   // Die Laenge der Datei ausgeben (zur Information des Benutzers):
   gotoXY(0, ZEILE_LAENGE);
   printf("Eingelesen wurden: %d Bytes", laenge);
   fflush(stdout);

   #ifdef TEST
   gotoXY(40, 1);
   printf("Laenge der Datei: %d Bytes", strlen(puffer));
   fflush(stdout);
   #endif // TEST

   // Kommando-Schleife: Wiederholt ein Kommando einlesen und ausfuehren,
   // bis der Benutzer ein 'q' eingibt:
   while (1) {             // wird durch exit in fuehreKommdoAus beendet
      gibSeiteAus(puffer, abHier);    // Eine Seite ausgeben

      // Ein Kommando ("su", "so", "zu" etc.) vom Benutzer einlesen:
      fgets(aktuellesKommando, 10, stdin);
      // Das newline-Zeichen am Ende entfernen:
      aktuellesKommando[strlen(aktuellesKommando)-1] = '\0';

      // Wenn der Benutzer nur auf Return gedrueckt hat, wird sein voriges
      // Kommando noch mal ausgefuehrt:
      if (strlen(aktuellesKommando) == 0) {
         strcpy(aktuellesKommando, vorigesKommando);
      }
      strcpy(vorigesKommando, aktuellesKommando);

      // aktuellesKommando ausfuehren (die Variable abHier
      // entsprechend veraendern, evtl. Programm beenden):
      fuehreKommandoAus(aktuellesKommando, &abHier, laenge);

   } // while (true)

   return 0;
} // main
/* ------------------------------------------------------------------------
Wiedergabe einer typischen Bilschirmausgabe des Programms Hexer02:

Hexer02: Pfadname einer Datei? Hexer02.c
Laenge der Datei: 15768 Bytes

     0: 2F 2F 20 44 61 74 65 69 20 48 65 78 65 72 30 32 //.Datei.Hexer02
    16: 2E 63 0A 2F 2A 20 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D .c./*.----------
    32: 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D ----------------
    48: 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D ----------------
    64: 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D ----------------
    80: 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 2D 0A 4C --------------.L
    96: 69 65 73 74 20 76 6F 6E 20 64 65 72 20 53 74 61 iest.von.der.Sta
   112: 6E 64 61 72 64 65 69 6E 67 61 62 65 20 64 65 6E ndardeingabe.den
   128: 20 50 66 61 64 6E 61 6D 65 6E 20 65 69 6E 65 72 .Pfadnamen.einer
   144: 20 44 61 74 65 69 20 44 20 65 69 6E 2C 20 6C 69 .Datei.D.ein,.li
   160: 65 73 74 20 64 69 65 0A 44 61 74 65 69 20 44 20 est.die.Datei.D.
   176: 75 6E 64 20 67 69 62 74 20 69 68 72 65 6E 20 49 und.gibt.ihren.I
   192: 6E 68 61 6C 74 20 69 6E 20 68 65 78 61 64 65 7A nhalt.in.hexadez
   208: 69 6D 61 6C 65 72 20 44 61 72 73 74 65 6C 6C 75 imaler.Darstellu
   224: 6E 67 20 75 6E 64 20 69 6E 20 7A 65 69 63 68 65 ng.und.in.zeiche
   240: 6E 2D 0A 77 65 69 73 65 72 20 44 61 72 73 74 65 n-.weiser.Darste
   256: 6C 6C 75 6E 67 20 7A 75 72 20 53 74 61 6E 64 61 llung.zur.Standa
Seite nach unten

Kommandos: zu, zo (Zeile nach unten/oben), nur Return (letzen Befehl nochmal)
           su, so (Seite nach unten/oben), q (quit)
           uu, oo (zum Ende/Anfang der Datei)Laenge der Datei: 15768 Bytes
------------------------------------------------------------------------ */
