// Datei Konsole.c
// ------------------------------------------------------------------------
// Implementierungsteil des Moduls Konsole
// ------------------------------------------------------------------------
#include "Konsole.h"
#include <windows.h>
#include <string.h>  // fuer strlen

HANDLE hStdout;
HANDLE hStdin;
//-------------------------------------------------------------------------
void initKonsole() {
   hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
   hStdin  = GetStdHandle(STD_INPUT_HANDLE);
} // init
//-------------------------------------------------------------------------
short AnzZeilen() {
   CONSOLE_SCREEN_BUFFER_INFO csbi;
   GetConsoleScreenBufferInfo(hStdout, &csbi);
   return csbi.dwSize.Y;
// return 25;
} // AnzZeilen;
//-------------------------------------------------------------------------
short AnzSpalten() {
   CONSOLE_SCREEN_BUFFER_INFO csbi;
   GetConsoleScreenBufferInfo(hStdout, &csbi);
   return csbi.dwSize.X;
// return 80;
} // AnzSpalten;
//-------------------------------------------------------------------------
BOOL ClearScreen(const COLOR vordergrund,  const COLOR hintergrund) {
   // 1. Fuer alle Zellen des Bildschirms werden Vordergrund- und Hinter-
   //    grundfarbe neu festgelegt.
   // 2. Die auf dem Bildschirm angezeigten Zeichen werden geloescht.
   // 3. Der Bildschirmzeiger (cursor) wird nach Hause geschickt (Zeile 0,
   //    Spalte 0).
   // 4. Die Farben fuer nachfolgende Ausgaben (zur Standardausgabe)
   //    werden festgelegt (entsprechend den Parametern vordergrund
   //    und hintergrund). Dies ist notwendig, um bestimmte kleine Unter-
   //    schiede zwischen Windows95 und WindowsNT auszugleichen.

   BOOL   ok = TRUE;
   DWORD soll_anzahl  = AnzZeilen() * AnzSpalten();
   DWORD ist_Anzahl; // Anz. der tatsaechlich gefuellten Zellen
   COORD ursprung     = {0, 0};
   WORD  farben       = vordergrund + 16 * hintergrund;
   const char  BLANK  = ' ';
   // ---------------------------------------------------------------------
   // Farbe aller Zellen neu festlegen:
   ok &= FillConsoleOutputAttribute(hStdout, farben, soll_anzahl,
      ursprung, &ist_Anzahl);

   // Die auf dem Bildschirm angezeigten Zeichen loeschen:
   ok &= FillConsoleOutputCharacter(hStdout, BLANK, soll_anzahl,
      ursprung, &ist_Anzahl);
   if (!ok) return FALSE;

   // Farben fuer nachfolgende Ausgaben festlegen:
   ok &= SetConsoleTextAttribute(hStdout, farben);

   // Bildschirmzeiger nach Hause schicken und Ergebnis liefern:
   ok &= GotoXY(0, 0);

   return ok;
} // ClearScreen
//-------------------------------------------------------------------------
BOOL SetColorsA(const COLOR  vordergrund, const COLOR  hintergrund) {
   WORD farben = vordergrund + 16 * hintergrund;
   return SetConsoleTextAttribute(hStdout, farben);
} // SetColors
//-------------------------------------------------------------------------
BOOL SetColorsE(const short spalte,
                const short zeile,
                const COLOR vordergrund,
                const COLOR hintergrund) {
   // Die Farben einer einzelnen Zelle des Bildschirms werden festgelegt:
   BOOL   ok;
   COORD  c;
   DWORD  anz;
   WORD   farben = vordergrund + 16 * hintergrund;
   // ---------------------------------------------------------------------
   // Die Parameter spalte und zeile pruefen:
   if ((spalte < 0) || (AnzSpalten() <= spalte)) return FALSE;
   if ((zeile  < 0) || (AnzZeilen()  <= zeile))  return FALSE;

   c.X = spalte;
   c.Y = zeile;
   ok  = SetConsoleTextAttribute(hStdout, farben);
   if (!ok) return FALSE;

   return WriteConsoleOutputAttribute(hStdout, &farben, 1, c, &anz);
} // SetColors1
//-------------------------------------------------------------------------
BOOL SetColorsV   (const short  von_spalte,
                   const short  bis_spalte,
                   const short  von_zeile,
                   const short  bis_zeile,
                   const COLOR  vordergrund,
                   const COLOR  hintergrund) {
   // Die Farben eines ganzen Rechtecks auf dem Bildschirm werden
   // festgelegt:

   BOOL  ok = TRUE;
   COORD c;
   DWORD anz;
   WORD  farben = vordergrund + 16 * hintergrund;
   // ---------------------------------------------------------------------
   // Die Parameter von_spalte, ..., bis_zeile pruefen:
   if ((von_spalte < 0) || (AnzSpalten() <= von_spalte)) return FALSE;
   if ((bis_spalte < 0) || (AnzSpalten() <= bis_spalte)) return FALSE;
   if ((von_zeile  < 0) || (AnzZeilen()  <= von_zeile )) return FALSE;
   if ((bis_zeile  < 0) || (AnzZeilen()  <= bis_zeile )) return FALSE;

   ok &= SetConsoleTextAttribute(hStdout, farben);

   for (c.Y = von_zeile; c.Y <= bis_zeile; c.Y++) {
      for (c.X = von_spalte; c.X <= bis_spalte; c.X++) {
         ok &= WriteConsoleOutputAttribute(hStdout, &farben, 1, c, &anz);
      }
   }
   return ok;
} // SetColors2
//-------------------------------------------------------------------------
BOOL PutS(const short spalte, const short zeile, const char *string) {
   DWORD laenge = strlen(string);
   COORD c;
   DWORD anz;
   // ---------------------------------------------------------------------
   c.X = spalte;
   c.Y = zeile;
   return WriteConsoleOutputCharacter(hStdout, string, laenge, c, &anz);
} // PutS
//-------------------------------------------------------------------------
BOOL PutC(const short spalte, const short zeile, const char zeichen) {
   char  text[2] = "?";
   DWORD laenge = 1;
   COORD c;
   DWORD anz;
   int   erg;
   // ---------------------------------------------------------------------
   c.X     = spalte;
   c.Y     = zeile;
   text[0] = zeichen; // In text[1] steht schon \0

   erg = WriteConsoleOutputCharacter(hStdout, text, laenge, c, &anz);
   return erg;
} // PutC
//-------------------------------------------------------------------------
BOOL GetImmediate(char * zeichen) {
   DWORD      modus_alt;
   DWORD      modus_neu = 0; // sofort Lesen ohne Return, kein Echo
   DWORD      anzahl_gelesen;
   BOOL       ok;
   const char CR  = 13;
   const char LF  = 10;
   const char NUL =  0;
   // ---------------------------------------------------------------------
   // Alten Modus der Konsole holen:
   if (!GetConsoleMode(hStdin, &modus_alt)) {
      return FALSE;
   }
   // Neuen Modus (ohne Echo und ohne Warten auf Return) setzen:
   if (!SetConsoleMode(hStdin, modus_neu)) {
      return FALSE;
   }
   // Ein Zeichen lesen (ohne Echo und ohne Warten auf Return).
   // Die Zeichen CR, LF und NUL werden "ueberlesen":
   do {
      ok = ReadConsoleA(hStdin, zeichen, 1, &anzahl_gelesen, NULL);
   } while (ok && (*zeichen == CR || *zeichen == LF || *zeichen == NUL));

   // Egal ob ReadConsoleA geklappt hat oder nicht: Den alten
   // Modus der Konsole wieder herstellen (hoffentlich klappt das):
   SetConsoleMode(hStdin, modus_alt); // Alten Modus wiederherstellen

   return ok;
} // GetImmediate
//-------------------------------------------------------------------------
BOOL GetImmediateAny(char  *z, short *vkc, int *cks) {
   INPUT_RECORD ip;
   DWORD        anzGelesen;
   BOOL         ok;
   // ---------------------------------------------------------------------
   while (TRUE) {                   // Schleife wird mit return verlassen
      ok = ReadConsoleInput(hStdin, &ip, 1, &anzGelesen);
      // Alle Ereignisse (events) ueberlesen ausser KeyDown-Ereignisse:
      if (ip.EventType != KEY_EVENT || !ip.Event.KeyEvent.bKeyDown) {
         continue;
      }
      *vkc = (short)(ip.Event.KeyEvent.wVirtualKeyCode);
//    *cks = (short)(ip.Event.KeyEvent.dwControlKeyState);
      *cks = (int)  (ip.Event.KeyEvent.dwControlKeyState);
      *z   =         ip.Event.KeyEvent.uChar.AsciiChar;
      break;
   } // while
   return ok;
}// GetImmediateAny
//-------------------------------------------------------------------------
// CAPSLOCK_ON, ENHANCED_KEY, ... SHIFT_PRESSED sind Makros (oder: #define-
// Konstanten), die in der Datei Defines.h definiert sind.
// Die Werte dieser Konstanten sind im folgenden als Kommentar angegeben.
// "cks" soll an "Control Key State" erinnern:
int Capslock_On       (int cks) {return cks & CAPSLOCK_ON;}        // 128
int Enhanced_Key      (int cks) {return cks & ENHANCED_KEY;}       // 256
int Left_Alt_Pressed  (int cks) {return cks & LEFT_ALT_PRESSED;}   //   2
int Left_Ctrl_Pressed (int cks) {return cks & LEFT_CTRL_PRESSED;}  //   8
int Numlock_On        (int cks) {return cks & NUMLOCK_ON;}         //  32
int Right_Alt_Pressed (int cks) {return cks & RIGHT_ALT_PRESSED;}  //   1
int Right_Ctrl_Pressed(int cks) {return cks & RIGHT_CTRL_PRESSED;} //   4
int Scrolllock_On     (int cks) {return cks & SCROLLLOCK_ON;}      //  64
int Shift_Pressed     (int cks) {return cks & SHIFT_PRESSED;}      //  16
//-------------------------------------------------------------------------
BOOL GotoXY(const short spalte, const short zeile) {
   COORD c;
   c.X = spalte;
   c.Y = zeile;
   return SetConsoleCursorPosition(hStdout, c);
} // GotoXY
//-------------------------------------------------------------------------
BOOL FenstergroesseMaximal() {
   COORD   max = GetLargestConsoleWindowSize(hStdout);
   return SetConsoleScreenBufferSize(hStdout, max);
} // FenstergroesseMaximal
//-------------------------------------------------------------------------
BOOL Fenstergroesse_80_25() {
   COORD   std = {80, 25};
   return SetConsoleScreenBufferSize(hStdout, std);
} // Fenstergroesse_80_25
//-------------------------------------------------------------------------
COLOR ColorSucc(const COLOR c) {
   return (c==BRIGHT_WHITE ? BLACK : (COLOR)(c + 1));
} // intSucc
//-------------------------------------------------------------------------
COLOR ColorPred(const COLOR c) {
   return (c==BLACK ? BRIGHT_WHITE : (COLOR)(c - 1));
} // intPred
//-------------------------------------------------------------------------
const char *ColorName(const COLOR c) {
   static const char *Namen[16] =
      {"BLACK",       "BLUE",           "GREEN",         "CYAN",
       "RED",         "MAGENTA",        "YELLOW",        "WHITE",
       "GRAY",        "BRIGHT_BLUE",    "BRIGHT_GREEN",  "BRIGHT_CYAN",
       "BRIGHT_RED",  "BRIGHT_MAGENTA", "BRIGHT_YELLOW", "BRIGHT_WHITE"
      };
  if (c < 0 || 15 < c) return "Falscher COLOR-Code!";
  return Namen[c];
} // intName
//-------------------------------------------------------------------------
