// Datei Konsole.h
/* ------------------------------------------------------------------------
Der Modul Konsole besteht aus den Dateien Konsole.h und Konsole.c.
Mit diesem Modul (und einem geeignet C-Compiler) kann man unter einem
Windows-Betriebssystem
- das Standardausgabefenster loeschen (ClearScreen)
- fuer das Standardausgabefenster Farben festlegen (SetColors)
- einzelne Zeichen und ganze Zeichenketten zu bestimmten Stellen des
  Standardausgabefensters ausgeben (Put)
- im Standardausgabefenster den Cursor positionieren (GotoXY)
- von der Standardeingabe einzelne Zeichen "sofort" und ohne Echo lesen,
  (d.h. der Benutzer muss nicht auf die Return-Taste druecken und das
  eingegebene Zeichen erscheint nicht auf dem Bildschirm (GetImmediate
  und GetImmediateAny).
- die Groesse des Standardausgabefensters veraendern
  (FenstergroesseMaximal und Fenstergroesse_80_25)
- zu einer Farbe zyklisch die nachfolgende Farbe bzw. die vorhergehende
  Farbe berechnen lassen (ColorSucc und ColorPred).
- sich den Namen einer Farbe (als C-String) liefern lassen (ColorName).

Die bool-Funktionen liefern TRUE, wenn "alles geklappt hat". Sie liefern
FALSE, wenn ein Fehler auftrat (z.B. weil die Parameter falsch waren).
------------------------------------------------------------------------ */
#ifndef Konsole_h
#define Konsole_h

#include <windows.h> // fuer den Typ BOOL
//-------------------------------------------------------------------------
// Diese Funktion muss einmal und vor allen anderen Funktionen dieses
// Moduls aufgerufen werden:
void initKonsole(void);

// Groesse des Standardausgabefensters:
short AnzZeilen (void); // Liefert die Anzahl der Zeilen
short AnzSpalten(void); // Liefert die Anzahl der Spalten
//-------------------------------------------------------------------------
// Eine Farbe ist eine 4-Bit-Binaerzahl, mit je einem Bit fuer RED, GREEN
// BLUE und BRIGHT. BRIGHT plus RED ist BRIGHT_RED, BLUE plus GREEN ist
// CYAN, BRIGHT plus BLUE plus GREEN ist BRIGHT_CYAN etc. Die Farbe
// bright black wird hier als GRAY bezeichnet:
enum Color {
   BLACK     = 0, BLUE          = 1, GREEN        = 2, CYAN        = 3,
   RED       = 4, MAGENTA       = 5, YELLOW       = 6, WHITE       = 7,
   GRAY      = 8, BRIGHT_BLUE   = 9, BRIGHT_GREEN =10, BRIGHT_CYAN =11,
   BRIGHT_RED=12, BRIGHT_MAGENTA=13, BRIGHT_YELLOW=14, BRIGHT_WHITE=15
};

#define COLOR enum Color
//-------------------------------------------------------------------------
BOOL ClearScreen (const COLOR  vordergrund,
                  const COLOR  hintergrund);
// Loescht den Bildschirm und schickt den Bildschirmzeiger (cursor) nach
// Hause (Spalte 0, Zeile 0). Wahlweise kann eine Farbe fuer den Vorder-
// grund und eine Farbe fuer den Hintergrund angegeben werden. Diese
// Farben gelten dann fuer alle Zellen des Standardausgabefensters und
// alle zukuenftigen Ausgaben zum Standardausgabefenster (bis zum naechsten
// Aufruf dieser Funktion ClearScreen oder einer SetColors-Funktion).
//-------------------------------------------------------------------------
BOOL SetColorsA  (const COLOR  vordergrund,
                  const COLOR  hintergrund);
// Legt die Farben fuer *zukuenftige* Ausgaben zum Standardausgabefenster
// fuer *alle* Zeichen-Zellen fest. Die Farben gelten nur fuer Ausgaben wie
// z.B. printf("Hallo!");, nicht aber fuer Ausgaben mit den Funktionen
// PutC und PutS, die in diesem Modul vereinbart werden (siehe unten).
// Die Farben bleiben wirksam bis zum naechsten Aufruf einer SetColors-
// Funktion oder der ClearScreen-Funktion. Ausgaben mit printf erscheinen
// moeglicherweise erst nach einem Aufruf wie fflush(stdout);
//-------------------------------------------------------------------------
BOOL SetColorsE  (const short  spalte,
                  const short  zeile,
                  const COLOR  vordergrund,
                  const COLOR  hintergrund);
// Legt die Hintergrundfarbe und die Vordergrundfarbe fuer *eine* Zeichen-
// Zelle des Standardausgabefensters fest (das Standardausgabefenster
// besteht aus AnzZeilen * AnzSpalten vielen Zeichen-Zellen).
//-------------------------------------------------------------------------
BOOL SetColorsV  (const short  von_spalte,
                  const short  bis_spalte,
                  const short  von_zeile,
                  const short  bis_zeile,
                  const COLOR  vordergrund,
                  const COLOR  hintergrund);
// Legt die Hintergrundfarbe und die Vordergrundfarbe fuer ein Rechteck
// auf dem Bildschirm (fuer *viele* Zeichen-Zellen) fest. Das Rechteck
// kann aus beliebig vielen Zeichen-Zellen bestehen (maximal aus allen
// Zellen des Bildschirms).
//-------------------------------------------------------------------------
BOOL PutS        (const short  spalte,
                  const short  zeile,
                  const char  *string);
// Gibt den string zum Standardausgabefenster aus. Das erste Zeichen des
// Textes wird in die Zelle (spalte, zeile) geschrieben.
// ACHTUNG: Die Position des Bildschirmzeigers (cursor) wird durch dieses
// Unterprogramm *nicht* veraendert!
//-------------------------------------------------------------------------
BOOL PutC        (const short  spalte,
                  const short  zeile,
                  const char   zeichen);
// Gibt das zeichen zum Standardausgabefenster aus, an die Position (spalte,
// zeile).
// ACHTUNG: Die Position des Bildschirmzeigers (cursor) wird durch dieses
// Unterprogramm *nicht* veraendert!
//-------------------------------------------------------------------------
BOOL GotoXY      (const short  spalte,
                  const short  zeile);
// Bringt den Bildschirmzeiger im Standardausgabefenster an die Position
// (spalte, zeile). Nachfolgende Ausgaben (z.B. puts("Hallo!";) erfolgen
// ab dieser neuen Position. Ausgaben mit printf erscheinen evtl. erst
// nach einem Aufruf wie fflush(stdout); .
//-------------------------------------------------------------------------
BOOL GetImmediateAny(char  *zeichen,
                     short *psVirtualKeyCode,
                     int   *piControlKeyState);
// Liest ein Zeichen von der Standardeingabe (Tastatur). Der eingebende
// Benutzer muss seine Eingabe *nicht* mit der Return-Taste abschliessen.
// Die Eingabe erfolgt *ohne Echo* (d.h. das eingegebene Zeichen erscheint
// nicht automatisch auf dem Bildschirm). Mit diesem Unterprogramm kann
// *jeder* Tastendruck ("any key") gelesen werden (auch ein Druck auf eine
// Taste wie Pfeil-nach-oben, Bild-nach-unten, Umschalt-Taste, linke
// Alt-Taste, Rechte Alt-Taste, linke Strg-Taste, rechte Strg-Taste oder
// auf eine Funktionstaste F1, F2 etc. etc.). Die folgenden Funktionen
// sollen das Analysieren des Parameters *piControlKeyState erleichtern.
// ------------------------------------------------------------------------
// Funktionen zur Ananlyse des Parameters *piControlKeyState der
// Funktion GetImmediateAny:
int Capslock_On       (int iControlKeyState);
int Enhanced_Key      (int iControlKeyState);
int Left_Alt_Pressed  (int iControlKeyState);
int Left_Ctrl_Pressed (int iControlKeyState);
int Numlock_On        (int iControlKeyState);
int Right_Alt_Pressed (int iControlKeyState);
int Right_Ctrl_Pressed(int iControlKeyState);
int Scrolllock_On     (int iControlKeyState);
int Shift_Pressed     (int iControlKeyState);
//-------------------------------------------------------------------------
BOOL GetImmediate(char  *zeichen);
// Liest ein Zeichen von der Standardeingabe (Tastatur). Der eingebende
// Benutzer muss seine Eingabe *nicht* mit der Return-Taste abschliessen.
// Die Eingabe erfolgt *ohne Echo* (d.h. das eingegebene Zeichen erscheint
// nicht automatisch auf dem Bildschirm). Mit diesem Unterprogramm koennen
// nur "normale Zeichen" (Buchstaben, Ziffern, Sonderzeichen etc.) einge-
// lesen werden. Steuerzeichen (z.B. Pfeil-nach-oben, Bild-nach-unten etc.)
// werden vom Betriebssystem "verschluckt".
//-------------------------------------------------------------------------
BOOL FenstergroesseMaximal();
// Vergroessert das Standardausgabefenster auf die maximale Groesse (die
// von der Groesse des Bildschirms und dem verwendeten Font abhaengt).
//-------------------------------------------------------------------------
BOOL Fenstergroesse_80_25();
// Veraendert die Groesse des Standardausgabefensters so, dass es 25 Zeilen
// mit je 80 Spalten enthaelt.
//-------------------------------------------------------------------------
COLOR ColorSucc(const COLOR c);
// Zyklische Nachfolgerfunktion fuer Color (nach BRIGHT_WHITE kommt BLACK)
COLOR ColorPred(const COLOR c);
// Zyklische Vorgaengerfunktion fuer Color (vor BLACK liegt BRIGHT_WHITE)
const char *ColorName(const COLOR c);
// Liefert den Namen der Farbe c als String.
//-------------------------------------------------------------------------
#endif // Konsole_h
