// Datei KonsoleKeys.c
/* ------------------------------------------------------------------------
Liest mit der Funktion GetImmediateAll() Zeichen von der Standardeingabe
(d.h. die Zeichen werden ohne Echo gelesen und ohne auf Return zu
warten) und gibt folgende Daten zur aktuellen Ausgabe aus:
- Das gelesene Zeichen (bzw. ein Blank, falls es nicht druckbar ist)
- Das gelesene Zeichen als Zahl (zwischen 0 und 255)
- Den (geraeteunabhaengigen) virtual key code des Zeichens
- Den control key status
- Die einzelnen Bits des control key status (Capslock_On, Enhanced_Key
  bis Shift_Pressed)
------------------------------------------------------------------------ */
#include "Konsole.h" // fuer ClearScreen, GetImmediateAny, GotoXY, ...
#include <stdio.h>

int main() {

   int           alles_ok;
   char          zeichen_c  = 'x'; // Anfangswert ungleich 'q'
   unsigned char zeichen_uc;
   int           zeichen_i;
   short         vkc;              // virtual key code
   int           cks;              // control key state

   #define              LEN  10
   char alsChar        [LEN];
   char alsInt         [LEN];
   char virtualKeyCode [LEN];
   char controlKeyState[LEN];
   char alles_ok_s     [LEN];

   char * capslock_on        ;
   char * enhanced_key       ;
   char * left_alt_pressed   ;
   char * left_ctrl_pressed  ;
   char * numlock_on         ;
   char * right_alt_pressed  ;
   char * right_ctrl_pressed ;
   char * scrolllock_on      ;
   char * shift_pressed      ;
   // ---------------------------------------------------------------------
   initKonsole();
   ClearScreen(BRIGHT_WHITE, BLACK);

   printf("Programm KonsoleKeys: Jetzt geht es los!\n");
   printf("Bitte Tasten druecken (q zum Beenden): \n");

   while (zeichen_c != 'q') {
      alles_ok   = GetImmediateAny(&zeichen_c, &vkc, &cks);
      zeichen_uc = (unsigned char)(zeichen_c);
      zeichen_i  = (int)          (zeichen_uc);

      // Nicht-druckbare Zeichen durch Punkt '.' ersetzen:
      if (zeichen_uc < 32) zeichen_c = '.';

      // Informationen ueber die gedrueckte(n) Tase(n) in Strings umwandeln:

      capslock_on        = Capslock_On       (cks) ? "on " : "off";
      enhanced_key       = Enhanced_Key      (cks) ? "on " : "off";
      left_alt_pressed   = Left_Alt_Pressed  (cks) ? "on " : "off";
      left_ctrl_pressed  = Left_Ctrl_Pressed (cks) ? "on " : "off";
      numlock_on         = Numlock_On        (cks) ? "on " : "off";
      right_alt_pressed  = Right_Alt_Pressed (cks) ? "on " : "off";
      right_ctrl_pressed = Right_Ctrl_Pressed(cks) ? "on " : "off";
      scrolllock_on      = Scrolllock_On     (cks) ? "on " : "off";
      shift_pressed      = Shift_Pressed     (cks) ? "on " : "off";

//    sprintf(alsChar,         "'%c'   ", zeichen_i);
//    sprintf(alsChar,         "'%c'   ", zeichen_uc);
      sprintf(alsChar,         "'%c'   ", zeichen_c);
      sprintf(alsInt,          "%d     ", zeichen_i);
      sprintf(virtualKeyCode,  "%hd    ", vkc);
      sprintf(controlKeyState, "0x%04x ", cks);
      sprintf(alles_ok_s,      "%d     ", alles_ok);

      // Informationen ueber die gedrueckte(n) Taste(n) ausgeben:
      PutS(0,  2, "Zeichen als char  : "); PutS(21,  2, alsChar);
      PutS(0,  3, "Zeichen als int   : "); PutS(21,  3, alsInt);
      PutS(0,  4, "VirtualKeyCode    : "); PutS(21,  4, virtualKeyCode);
      PutS(0,  5, "ControlKeyState   : "); PutS(21,  5, controlKeyState);
      PutS(0,  6, "CapsLock          : "); PutS(21,  6, capslock_on);
      PutS(0,  7, "Enhanced_Key      : "); PutS(21,  7, enhanced_key);
      PutS(0,  8, "Left_Alt_Pressed  : "); PutS(21,  8, left_alt_pressed);
      PutS(0,  9, "Left_Ctrl_Pressed : "); PutS(21,  9, left_ctrl_pressed);
      PutS(0, 10, "Numlock_On        : "); PutS(21, 10, numlock_on);
      PutS(0, 11, "Right_Alt_Pressed : "); PutS(21, 11, right_alt_pressed);
      PutS(0, 12, "Right_Ctrl_Pressed: "); PutS(21, 12, right_ctrl_pressed);
      PutS(0, 13, "Scrolllock_On     : "); PutS(21, 13, scrolllock_on);
      PutS(0, 14, "Shift_Pressed     : "); PutS(21, 14, shift_pressed);
      PutS(0, 15, "Alles O.K.        : "); PutS(21, 15, alles_ok_s);
   } // while

   ClearScreen(WHITE, BLACK);
   printf("Programm KonsoleKeys: Das war's erstmal!\n");

   return 0;
} // main
// ------------------------------------------------------------------------
