// Datei Turtle1.c
/* ------------------------------------------------------------------------
Ein einfaches Schildkroeten-Grafik-Programm (turtle graphics program).Mit
den Pfeiltasten kann man auf dem Standardausgabebildschirm "Spuren einer
Schildkroete" zeichen. Durch Eingabe von 'u' (wie up) kann man die Schild-
kroete "hochheben", so dass sie keine Spuren mehr hinterlaesst. Durch
Eingabe von 'd' (wie down) kann man sie wieder "runterlassen". Die Eingabe
von 'c' loescht den Bildschirm, mit 'f' kann man die Farbe der Spur
wechseln und jedes andere druckbare Zeichen wird als neues "Spurzeichen"
genommen.
---------------------------------------------------------------------------
Achtung: Falls Numlock an ist, liefern alle Pfeiltasten den gleichen vkc
(virtual key code). Desshalb muss Numlock ausgeschaltet werden, bevor die
Pfeiltasten benutzt werden.
---------------------------------------------------------------------------
Hinweis: Der vkc (virtual key code) eines kleinen oder grossen Buchstabens
ist gleich dem ASCII-Code des grossen Buchstabens. Gibt der Benutzer z.B.
ein 'a' oder ein 'A' ein, ist der vkc gleich 'A' (dezimal 65).
------------------------------------------------------------------------ */
#include "Konsole.h"
#include <ctype.h>   // fuer iscntrl()
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#define true 1
// ------------------------------------------------------------------------
void zeichne(short x, short y, char zeichen, int vgf, int hgf) {
   // Gibt das zeichen aus, falls der Stift nicht "hoch" ist (d.h. falls
   // zeichen ungleich 0 ist):
   if (zeichen != 0) PutC(x, y, zeichen);
   SetColorsE(x, y, vgf, hgf);
   GotoXY    (x, y);
} // zeichne
// ------------------------------------------------------------------------
int main() {
   short        x;
   short        y;
   char         c;                            // Wird eingelesen
   short        vkc;                          // virtual key code
   int          cks;                          // control key state
   char         aktSpur  = 'X';               // aktuelle Spur
   char         altSpur;                      // vorige   Spur
   char const * WARNUNG  = "Bitte auf NUM-Taste druecken!";
   char const * BLANKS   = "                             ";
   int          vgf      = BLACK;             // Vordergrundfarbe
   int          hgf      = BRIGHT_WHITE;      // Hintergrundfarbe

   initKonsole();
   x = AnzSpalten() / 2;
   y = AnzZeilen () / 2;

   // Am Anfang ein Zeichen in die Mitte des Bildschirms ausgeben:
   ClearScreen(BLACK, BRIGHT_WHITE);

   zeichne(x, y, aktSpur, vgf, hgf);

   // Zeichen einlesen und entsprechende Aktion ausfuehren bis der
   // Benutzer 'q' oder 'Q' eingibt:
   while (true) {
      GetImmediateAny(&c, &vkc, &cks);

      // Falls Numlock eingeschaltet ist, eine Warnung ausgeben. Muss
      // schon hier (*vor* der switch-Anweisung) erfolgen, weil Numlock
      // schon vor dem Starten dieses Programms eingeschaltet worden sein
      // koennte, so dass das Programm den Numlock-vkc 144 des Einschalt-
      // vorgangs nicht zu sehen bekam:
      if (Numlock_On(cks)) {PutS(0, 0, WARNUNG); continue; }

      switch (vkc) {
         // Wenn Numlock gerade ausgeschaltet wurde, WARNUNG loeschen:
         case 144: if (!Numlock_On(cks)) PutS(0, 0, BLANKS); break;
         // Wenn eine Pfeiltaste gedrueckt wurde, dann um eins nach links,
         // rauf, rechts bzw. runter und aktSpur zeichnen:
         case  37: zeichne(--x,   y, aktSpur, vgf, hgf); break; // Eins nach links
         case  38: zeichne(  x, --y, aktSpur, vgf, hgf); break; // Eins nach unten
         case  39: zeichne(++x,   y, aktSpur, vgf, hgf); break; // Eins nach rechts
         case  40: zeichne(  x, ++y, aktSpur, vgf, hgf); break; // Eins nach oben

         // Wenn "Befehls-Buchstabe" eingegeben wurde:
         case 'C':                     // Wenn Eingabe 'c' oder 'C' dann
            ClearScreen(BRIGHT_WHITE, BLACK);
            x = AnzSpalten() / 2;      // Zurueck in die
            y = AnzZeilen () / 2;      // Mitte des Bildschirms
            zeichne(x, y, aktSpur, vgf, hgf);
            break;
         case 'U':                     // Wenn Eingabe 'u' oder 'U' dann
            altSpur = aktSpur;         // up,   Zeichenstift hoch
            aktSpur = 0;
            break;
         case 'D':                     // Wenn Eingabe 'd' oder 'D' dann
            aktSpur = altSpur;         // down, Zeichenstift runter
            break;
         case 'F':                     // Wenn Eingabe 'f' oder 'F' dann
            vgf = ColorSucc(vgf);      // Vordergrund-Farbe aendern
            zeichne(x, y, aktSpur, vgf, hgf);
            break;
         case 'Q':                     // Wenn Eingabe 'q' oder 'Q' dann
            ClearScreen(BRIGHT_WHITE, BLACK);
            goto Ende;                 // quit, Programm beenden

         // Andere virtual key codes:
         default:
            if (!iscntrl(c)) {         // Steuerzeichen ignorieren.
               aktSpur = c;            // Anderes Zeichen als neues
               zeichne(x, y, aktSpur, vgf, hgf); // Spurzeichen verwenden.
            }
            break;
      } // switch (vkc)
   } // while (true)
   Ende: printf("Turtle1: Das war's erstmal!\n");
   return 0;
} // main
// ------------------------------------------------------------------------
