// Datei scanfInt.c
/* ------------------------------------------------------------------------
Programm zum Kennenlernen des scanf-Befehls.

In einer aeusseren Schleife wird jeweils ein Formatstring fuer einen
scanf-Befehl eingelesen.

In einer inneren Schleife werden dann mit einem scanf-Befehl und
diesem Formatstring jeweils zwei Werte in zwei Variablen var1 und var2
(vom Typ int) eingelesen und anschliessend wieder ausgegeben.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <string.h>  // memcopy, memmove, strcpy, strcat, ...
//#include <stdbool.h> // Makros bool, true, false, ...

// Eingelesen werden Werte in Variablen des Typs TYP:
#define TYP int
// Maximale Laenge des Formatstrings fs:
#define MAXLEN 50
// Weil der Borland-C/C++-Compiler bcc32 stdbool.h nicht kennt:
#define true    1
// ------------------------------------------------------------------------
int main() {
   // Die Format-String zum Ausgeben der eingelesenen Werte:
   const char *fsa1 = "Wert von var1: %d\n";
   const char *fsa2 = "Wert von var2: %d\n";

   // Beispiele fuer Formatstrings, die der Benutzer eingeben kann:
   const char *b1   = "%d %d";
   const char *b2   = "%3d %d";
   const char *b3   = "%x %o";
   const char *b4   = "%d , %d";
   const char *b5   = "q zum Beenden:";

   // Meldungstext der in der inneren Schleife ausgegeben wird:
   const char *text = "Bitte 2 Ganzzahlen eingeben (q zum Beenden): ";

   // Drei Parameter fuer den scanf-Befehl
         char fse  [MAXLEN];  // Formatstring, wird eingelesen
         TYP  var1;           // Variable, in sie wird eingelesen
         TYP  var2;           // Variable, in sie wird eingelesen

   // Puffer zum Einlesen von Daten, Muell (newline-Zeichen) und
   // einem einzelnen Zeichen ('q' fuer quit):
         char daten[MAXLEN];
         char muell[MAXLEN];
         int  naechstesZeichen;

   printf("scanfInt: Jetzt geht es los!\n");

   // Die aeussere Schleife dient zum Einlesen von Formatstrings:
   while (true) {
      printf("--------------------------------------------------------\n");
      printf("Mit dem folgenden scanf-Befehl sollen zwei Werte in die \n");
      printf("Variablen var1 und var2 (vom Typ int) gelesen werden:   \n");
      printf("----------------------------------                      \n");
      printf("scanf(fse, &var1, &var2);                               \n");
      printf("----------------------------------                      \n");
      printf("Bitte geben Sie zuerst einen Wert fuer den Formatstring \n");
      printf("fse ein. Moegliche Werte fuer fse sind z.B.:            \n");
      printf("%s oder %s oder %s oder %s oder %s\n", b1, b2, b3, b4, b5);
      printf("fse = ");

      // Soll diese Schleife (und damit das Programm) beendet werden?
      naechstesZeichen = getc(stdin);
      if (naechstesZeichen == 'q') {
         break;
      } else {
         ungetc(naechstesZeichen, stdin);
      } // if

      fgets(fse, MAXLEN, stdin); // Eine ganze Zeile als Formatstring lesen
      fse[strlen(fse)-1] = '\0';  // Das newline-Zeichen loeschen

      printf("----------------------------------\n");
      printf("Ausgefuehrt wird der scanf-Befehl:\n");
      printf("scanf(\"%s\", &var1, &var2);\n", fse);
      printf("----------------------------------\n");

      // Die innere Schleife dient zum Einlesen von jeweils zwei Werten:
      while (true) {

         // Die Variablen var1 und var2 mit einem "leicht erkennbaren" Wert
         // initialisieren (fuer den Fall, dass der scanf-Befehl ihnen
         // keinen Wert zuweist):
         var1 = 123456789;
         var2 = 123456789;

         printf(text); // Eingabeaufforderung (abhaengig vom Typ)

         // Soll diese Schleife beendet werden?
         naechstesZeichen = getc(stdin);
         if (naechstesZeichen == 'q') {
            fgets(muell, MAXLEN, stdin); // newline-Zeichen lesen
            break;
         } else {
            ungetc(naechstesZeichen, stdin);
         } // if

         // Der Benutzer sollte 2 Bruchzahlen so eingeben wie der
         // Formatstring fse es vorschreibt:
         scanf (fse, &var1, &var2);

         // Der Benutzer hat seine letzte Eingabe mit Return abgeschlossen,
         // aber der scanf-Befehl hat das newline-Zeichen noch NICHT aus
         // dem Eingabepuffer gelesen. Jetzt wird es gelesen:
         fgets(muell, MAXLEN, stdin);

         printf(fsa1, var1);
         printf(fsa2, var2);

      } // while
   } // while

   printf("--------------------------------------------------------\n");
   printf("scanfInt: Das war's erstmal!\n");
   return 0;
} // main
