// Datei Signale01.c
/* ------------------------------------------------------------------------
Ein Signal wird "abgefangen und behandelt". Liest wiederholt 2 Ganzzahlen
ein und gibt ihren Quotienten aus. Wenn der eingegebene Divisor gleich
0 ist, schickt der Ausfuehrer ein SIGFPE-Signal (arithmetischer Fehler).
In einem Behandler fuer SIGFPE-Signale wird der Divisor daraufhin auf 1
gesetzt und das Programm wird fortgesetzt (bis der Benutzer 'q' eingibt).
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <signal.h>  // Signale ("Ausnahmen") senden und fangen
// ------------------------------------------------------------------------
int dend = 7;        // Dividend
int dor  = 2;        // Divisor (wird im SIGFPE_Behandler veraendert)
int quot;            // Quotient
// ------------------------------------------------------------------------
void SIGFPE_Behandler(int sigNr) {
   // Diese Funktion wird als Behandler fuer SIGFPE-Signale angemeldet
   // und danach immer aufgerufen, wenn ein SIGFPE-Signal eintrifft:
   printf("\n");
   printf("--------------------------------------------------\n");
   printf("Der Behandler fuer SIGFPE-Signale wurde aufgerufen\n");
   printf("und hat den Divisor von %d auf 1 gesetzt!\n", dor);
   printf("--------------------------------------------------\n");

   dor  = 1; // Den Divisor korrigieren
   printf("%5d / %-5d ist gleich ", dend, dor);

   return;   // und an die "Fehlerstelle" zurueckkehren
} // SIGFPE_Behandler
// ------------------------------------------------------------------------
void liesUndRechne(void) {
   // Liest wiederholt zwei Ganzzahlen ein und gibt ihren Quotienten aus.
   // Dadurch wird moeglicherweis ein SIGFPE-Signal ausgeloest.
   #define LEN 50
    // Zum Lesen von 'q' und zum Ueberlesen von newline-Zeichen:
   char rest[LEN] = "x"; // Ungleich 'q'!

   while (rest[0] != 'q') {
      printf("%5d / %-5d ist gleich ", dend, dor);
      quot = dend / dor;
      printf("%-5d ", quot);
      printf("Zwei Ganzzahlen? ");
      scanf ("%d %d", &dend, &dor);
      fgets(rest, LEN, stdin); // newline und evtl. 'q' lesen
   }
} // liesUndRechne
// ------------------------------------------------------------------------
int main(void) {
   printf("Signale01: Jetzt geht es los!\n");

   // Eine Funktion zur Behandlung von SIGFPE-Signalen anmelden:
   signal(SIGFPE, SIGFPE_Behandler);

   // "Gefaehrliche Berechnungen" durchfuehren, die evtl. ein
   // SIGFPE-Signal ausloesen.
   liesUndRechne();

   printf("Signale01: Das war's erstmal!\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Ein Dialog mit dem Programm Signale01:

Signale01: Jetzt geht es los!
    7 / 2     ist gleich 3     Zwei Ganzzahlen? 17 3
   17 / 3     ist gleich 5     Zwei Ganzzahlen? 25 4
   25 / 4     ist gleich 6     Zwei Ganzzahlen? 333 0
  333 / 0     ist gleich
--------------------------------------------------
Der Behandler fuer SIGFPE-Signale wurde aufgerufen
und hat den Divisor von 0 auf 1 gesetzt!
--------------------------------------------------
  333 / 1     ist gleich 333   Zwei Ganzzahlen? 22 11
   22 / 11    ist gleich 2     Zwei Ganzzahlen? 789 13
  789 / 13    ist gleich 60    Zwei Ganzzahlen? 5 0
    5 / 0     ist gleich
--------------------------------------------------
Der Behandler fuer SIGFPE-Signale wurde aufgerufen
und hat den Divisor von 0 auf 1 gesetzt!
--------------------------------------------------
    5 / 1     ist gleich 5     Zwei Ganzzahlen? 100 17
  100 / 17    ist gleich 5     Zwei Ganzzahlen? q
Signale01: Das war's erstmal!
---------------------------------------------------------------------------
Dieser Dialog ist moeglich, nachdem man das Programm Signale01 mit dem
Gnu-gcc oder dem Borland-bcc32 erzeugt hat.

Erzeugt man das Programm dagegen mit dem lcc-win32, dann wird es
nach der ersten Ausfuehrung der Funktion SIGFPE_Behandler beendet (und
nicht mit dem korrigierten Divisor fortgesetzt).
------------------------------------------------------------------------ */
