// Datei Signale03.c

#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <signal.h>  // Signale ("Ausnahmen") senden und fangen
#include <setjmp.h>  // Lange Rueckspruenge aus einer Funktion hinaus
// ------------------------------------------------------------------------
const char * sigName(int sigNr) {
   // Wandelt sigNr in den Namen des Signals um. Unbekannte Nummern
   // werden in den Namen "???????" umgewandelt.
   switch (sigNr) {
      case SIGSEGV: return "SIGSEGV";
      case SIGFPE:  return "SIGFPE";
      case SIGINT:  return "SIGINT";
      case SIGABRT: return "SIGABRT";
      default:      return "???????";
   }
} // sigName
// ------------------------------------------------------------------------
// Die Befehle setjmp und longjmp brauchen einen Puffer:
jmp_buf programmZustand;
// ------------------------------------------------------------------------
void beHandlerA(int sigNr) {
   printf("--------------------------------------\n");
   printf("Der beHandlerA wurde aufgerufen mit der\n");
   printf("Signal-Nr. %d (%s)!\n", sigNr, sigName(sigNr));

   longjmp(programmZustand, sigNr);
} // beHandlerA
// ------------------------------------------------------------------------
void beHandlerB(int sigNr) {
   printf("\n--------------------------------------\n");
   printf("Der beHandlerB wurde aufgerufen mit der\n");
   printf("Signal-Nr. %d (%s)!\n", sigNr, sigName(sigNr));

   longjmp(programmZustand, sigNr);
} // beHandlerB
// ------------------------------------------------------------------------
void fuehreBerechnungenDurch(void) {
   float f01;
   float f02;
   int   n01=17, n02=9;

   while (1) {
      printf("Zwei Bruchzahlen (99 zum Beenden): ");
      scanf ("%f", &f01);
      if (f01 == 99.0) break;
      scanf ("%f", &f02);
      printf("Quotient: %f\n", f01 / f02); // Gleitpunktrechnung
      printf(" 17 / 0 : %d\n", n01 / n02); // Ganzzahlrechnung
   }
} // fuehreBerechnungenDurch
// ------------------------------------------------------------------------
void fuehreSpeicherzugriffeDurch(void) {
   int    n;
   int    m;
   char * adr01;
   char * text = "ABCD1234abcd";

   while (1) {
      printf("Eine korrekte Adresse: %p!\n", text);
      printf("Eine Adresse (99 zum Beenden)? ");
      scanf ("%p", &n);
      if (n == 0x99) break;
      adr01 = (char *) n;
      m     = *adr01;
      printf("An der Adresse %p steht %d!\n", adr01, m);
   }
} // fuehreSpeicherzugriffeDurch
// ------------------------------------------------------------------------
void signalBehandlerAnmelden(void) {
   void (* alterSEGV_Behandler)() = signal(SIGSEGV, beHandlerA);
   void (* alterFPE__Behandler)() = signal(SIGFPE,  beHandlerA);
   void (* alterINT__Behandler)() = signal(SIGINT,  beHandlerB);
   void (* alterABRT_Behandler)() = signal(SIGABRT, beHandlerB);

   if (alterSEGV_Behandler == SIG_ERR) {
      printf("Konnte keinen SIGSEGV-Behandler anmelden!\n");
   }

   if (alterFPE__Behandler == SIG_ERR) {
      printf("Konnte keinen SIGFPE-Behandler anmelden!\n");
   }

   if (alterINT__Behandler == SIG_ERR) {
         printf("Konnte keinen SIGINT-Behandler anmelden!\n");
   }

   if (alterABRT_Behandler == SIG_ERR) {
         printf("Konnte keinen SIGABRT-Behandler anmelden!\n");
   }
} // signalBehandlerAnmelden
// ------------------------------------------------------------------------
int main(void) {
   int setjmpErg;

   printf("Signale03: Jetzt geht es los!\n");

   signalBehandlerAnmelden();

   // Hier einen Ruecksprungspunkt fuer longjmp festlegen:
   setjmpErg = setjmp(programmZustand);

   if (setjmpErg == 0) {
      printf("========================================\n");
      printf("A Der setjmp-Puffer wurde initialisiert!\n");
   } else {
      printf("--------------------------------------\n");
      printf("B Rueckkehr mit longjmp, Param: %d\n", setjmpErg);
   }

   // Die Behandler-Funktionen enden alle mit einem longjmp-Befehl:
   signalBehandlerAnmelden();

   // Gefaehrliche Befehle ausfuehren, bei denen Signale (z.B.
   // SIGSEGV) gesendet werden koennen:
   fuehreSpeicherzugriffeDurch();

   // Hier einen anderen Ruecksprungspunkt fuer longjmp festlegen:
   setjmpErg = setjmp(programmZustand);

   if (setjmpErg == 0) {
      printf("========================================\n");
      printf("C Der setjmp-Puffer wurde initialisiert!\n");
   } else {
      printf("--------------------------------------\n");
      printf("D Rueckkehr mit longjmp, Param: %d\n", setjmpErg);
   }

   // Die Behandler-Funktionen enden alle mit einem longjmp-Befehl:
   signalBehandlerAnmelden();

   // Gefaehrliche Befehle ausfuehren, bei denen Signale (z.B.
   // SIGFPE) gesendet werden koennen:
   fuehreBerechnungenDurch();

   printf("Signale03: Das war's erstmal!\n");

   return 0;
} // main
