// Datei Verbunde01.c
/* ------------------------------------------------------------------------
Verbundtypen (struct types) werden deklariert und auf verschiedene Weise
mit Namen versehen. Variablen dieser Typen werden definiert und initiali-
siert. Funktionen mit Parametern dieser Typen und eine Funktion mit
einem Verbundtyp als Rueckgabetyp (Ergebnistyp) werden definiert und
aufgerufen.
------------------------------------------------------------------------ */
#include <stdio.h>  // fuer printf
// ------------------------------------------------------------------------
struct Person {                     // Der Typ heisst "struct Person"
   unsigned short jahre;
   unsigned short zentimeter;
   float          kilo;
};

struct Person p01 = {23, 175, 72.5};
struct Person p02 = {12, 142, 51.0};
// ------------------------------------------------------------------------
struct Bruch {                      // Der Typ heisst "struct Bruch"
   int      zaehler;
   unsigned nenner;
} b01;                              // "b01" bezeichnet eine Variable!

struct Bruch b02 = {-123, 17};
struct Bruch b03 = {3333, 5};
// ------------------------------------------------------------------------
typedef struct {
   char * vor;
   char * nach;
} NAME;                             // "NAME" bezeichnet einen Typ!

NAME n01 = {"Karl", "Schmitz"};
NAME n02 = {"Ali",  "Oezdemir"};
// ------------------------------------------------------------------------
typedef struct Komplex {            // Der Typ heisst "struct Komplex" und
   double re;
   double im;
} KOMPLEX;                          // zusaetzlich auch "KOMPLEX"

struct Komplex k01 = {1.0, 2.0};
struct Komplex k02 = {2.5, 1.5};

KOMPLEX        k03 = {3.333, 6.666};
KOMPLEX        k04;
// ------------------------------------------------------------------------
// Jetzt wird ein Verbundtyp deklariert, der keinen Namen hat und mit dem
// man deshalb auch nichts anfangen kann. Einige C-Compiler erlauben diese
// Deklaration, andere warnen oder melden einen Fehler:
/*
struct {
   int   k1;
   float k2;
};
*/
// ------------------------------------------------------------------------
// Einige C-Compiler erlauben Verbunde mit 0 Komponenten (z.B. der
// Gnu-C-Compiler), andere erlauben sie nicht (z.B. bcc32 und lcc-win32):
/*
struct Leer1 {};
struct Leer2 {};

struct Leer1 l01;
struct Leer2 l02 = l01; // Dies ist auf jeden Fall ein Typfehler!
*/
// ------------------------------------------------------------------------
// void-Funktionen (Prozeduren) mit Parametern eines Verbundtyps:
void gibAusPerson(struct Person p) {
   printf("D Jahre: %3d, Zentimeter: %3d, Gewicht: %4.1f\n",
             p.jahre,    p.zentimeter,    p.kilo);
} // gibAusPerson
// ------------------------------------------------------------------------
void gibAusBruch(struct Bruch b) {
   printf("G %+10d/%d\n", b.zaehler, b.nenner);
} // gibAusBruch
// ------------------------------------------------------------------------
void gibAusName(NAME nam) {
   printf("J Nachname: %s, Vorname: %s\n", nam.nach, nam.vor);
} // gibAusName
// ------------------------------------------------------------------------
void gibAusKomplex(KOMPLEX k) {
   printf("M %10.2f + %-4.2f i\n", k.re, k.im);
} // gibAusKomplex
// ------------------------------------------------------------------------
// Eine Funktion mit einem Ergebnis eines Verbundtyps (und einem Parameter
// des selben Verbundtyps):
struct Person jahrePlus1(struct Person p) {
   p.jahre++;
   return p;
} // jahrePlus1
// ------------------------------------------------------------------------
int main() {
   printf("A Verbunde01: Jetzt geht es los!\n");
   printf("---------------------------------------------------------\n");
   printf("B sizeof(struct Person): %2d Bytes\n", sizeof(struct Person));
   printf("C sizeof(p01):           %2d Bytes\n", sizeof(p01));
   gibAusPerson(p01);
   gibAusPerson(p02);
   gibAusPerson(jahrePlus1(p02));
   printf("---------------------------------------------------------\n");
   printf("E sizeof(struct Bruch):  %2d Bytes\n", sizeof(struct Bruch));
   printf("F sizeof(b01):           %2d Bytes\n", sizeof(b01));
   gibAusBruch(b01);
   gibAusBruch(b02);
   gibAusBruch(b03);
   printf("---------------------------------------------------------\n");
   printf("H sizeof(NAME):          %2d Bytes\n", sizeof(NAME));
   printf("I sizeof(n01):           %2d Bytes\n", sizeof(n01));
   gibAusName(n01);
   gibAusName(n02);
   printf("---------------------------------------------------------\n");
   printf("K sizeof(KOMPLEX):       %2d Bytes\n", sizeof(KOMPLEX));
   printf("L sizeof(k01):           %2d Bytes\n", sizeof(k01));
   gibAusKomplex(k01);
   gibAusKomplex(k02);
   gibAusKomplex(k03);
   gibAusKomplex(k04);
   printf("---------------------------------------------------------\n");
   printf("N Verbunde01: Das war's erstmal!\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Ausgabe des Programms Verbunde01:

A Verbunde01: Jetzt geht es los!
---------------------------------------------------------
B sizeof(struct Person):  8 Bytes
C sizeof(p01):            8 Bytes
D Jahre:  23, Zentimeter: 175, Gewicht: 72.5
D Jahre:  12, Zentimeter: 142, Gewicht: 51.0
D Jahre:  13, Zentimeter: 142, Gewicht: 51.0
---------------------------------------------------------
E sizeof(struct Bruch):   8 Bytes
F sizeof(b01):            8 Bytes
G         +0/0
G       -123/17
G      +3333/5
---------------------------------------------------------
H sizeof(NAME):           8 Bytes
I sizeof(n01):            8 Bytes
J Nachname: Schmitz, Vorname: Karl
J Nachname: Oezdemir, Vorname: Ali
---------------------------------------------------------
K sizeof(KOMPLEX):       16 Bytes
L sizeof(k01):           16 Bytes
M       1.00 + 2.00 i
M       2.50 + 1.50 i
M       3.33 + 6.67 i
M       0.00 + 0.00 i
---------------------------------------------------------
N Verbunde01: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldung des C-Compilers lcc-win32, wenn die Typdeklaration
in den Zeilen 51 bis 54 nicht auskommentiert ist:

Error verbunde01.c: 54  empty declaration
---------------------------------------------------------------------------
Warnung des Gnu-C-Compilers gcc, wenn die Typdeklaration
in den Zeilen 51 bis 54 nich auskommentiert ist:

Verbunde01.c:54: warning: unnamed struct/union that defines no instances
---------------------------------------------------------------------------
Der Borland-C/C++-Compiler bcc32 akzeptiert die Typdeklaration
in den Zeilen 51 bis 54 "ohne zu murren".
------------------------------------------------------------------------ */
