// Datei Verbunde02.c
/* ------------------------------------------------------------------------
Ein Verbundtyp VIEL_TEXT, dessen Variablen sehr gross sind, wird deklariert.
Mit der Funktion VIEL_TEXT_K kann man Variablen dieses Typs erzeugen lassen.
In eine Schleife werden viele Variablen dieses Typs erzeugt und gleich
wieder an die Speicherverwaltung zurueckgegeben (mit free).
Dann werden viele Variablen dieses Typs erzeugt ohne sie wieder an die
Speicherverwaltung zurueckzugeben, bis das Programm (wegen Speichermangels)
abgebrochen wird.

Dieses Programm soll illustrieren, dass es in C (anders als in Java)
keine automatische Speicherbereinigung (garbage collection) gibt.
------------------------------------------------------------------------ */
#include <stdio.h>  // fuer printf
#include <stdlib.h> // fuer malloc, calloc, free
#include <string.h> // memcopy, memmove, strcpy, strcat, ...

// Jede Variable des Typs VIEL_TEXT enthaelt drei Komponenten der
// Laenge LEN Bytes. Von diesen Variablen werden ANZ viele erzeugt:
#define LEN 1000 * 1000 * 40
#define ANZ 20
// ------------------------------------------------------------------------
// Jede Variable des Typs VIEL_TEXT belegt ziemlich viel Speicher:
typedef struct {
   char text1[LEN];
   char text2[LEN];
   char text3[LEN];
} VIEL_TEXT;
// ------------------------------------------------------------------------
// Eine Art Konstruktor VIEL_TEXT_K fuer den Verbundtyp VIEL_TEXT (er
// liefert einen Zeiger auf einen neuen VIEL_TEXT-Verbund):
VIEL_TEXT * VIEL_TEXT_K(char text1[], char text2[], char text3[]) {
   int    i;
   char * ind;

   // Speicherplatz fuer eine neue VIEL_TEXT-Variable reservieren:
   VIEL_TEXT * erg = (VIEL_TEXT *) malloc(sizeof(VIEL_TEXT));

   // In jedes 10. Byte der neuen Variablen das Zeichen '\0' schreiben:
   ind = (char *) erg;
   for (i=0; i<sizeof(VIEL_TEXT); i+=10) ind[i]='\0';

   // Ein paar (vermutlich kurze) Texte in die Komponenten der neuen
   // Verbundvariable schreiben:
   strcpy(erg->text1, text1);
   strcpy(erg->text2, text2);
   strcpy(erg->text3, text3);

   return erg;
} // VIEL_TEXT
// ------------------------------------------------------------------------
void gibAusVIEL_TEXT(VIEL_TEXT * vtp) {
   printf("-----------------------------------------------------------\n");
   printf("text1: %s\n", vtp->text1);
   printf("text2: %s\n", vtp->text2);
   printf("text3: %s\n", vtp->text3);
} // gibAusVIEL_TEXT
// ------------------------------------------------------------------------
int main() {

   // Index-Variablen fuer for-Schleifen:
   int         i1, i2;

   // Eine Reihung von Zeigern. Jeder kann auf eine Variable des
   // Verbundtyps VIEL_TEXT zeigen:
   VIEL_TEXT * tab[ANZ];

   printf("-----------------------------------------------------------\n");
   printf("Verbunde02: Jetzt geht es los!\n");
   printf("-----------------------------------------------------------\n");
   printf("sizeof(VIEL_TEXT): %d\n", sizeof(VIEL_TEXT));
   printf("-----------------------------------------------------------\n");

   // Schleife 1: Variablen erzeugen und wieder freigeben (mit free):
   printf("i1: ");
   for (i1=0; i1<ANZ; i1++) {
      tab[0] = VIEL_TEXT_K("Hallo!", "Wie geht's?", "Danke, gut!");
      printf("%d ", i1);
      fflush(stdout);
      free(tab[0]);
   }
   printf("\n-----------------------------------------------------------\n");

   // Schleife 2: Variablen erzeugen und nicht wieder freigeben:
   printf("i2: ");
   for (i2=0; i2<ANZ; i2++) {
      tab[i2] = VIEL_TEXT_K("Hallo!", "Wie geht's?", "Danke, gut!");
      printf("%d ", i2);
      fflush(stdout);
//    free(tab[i2]);
   }
   printf("\n-----------------------------------------------------------\n");

   printf("Verbunde02: Das war's erstmal!\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Wenn man dieses Programm mit dem Gnu-C-Compiler gcc compiliert und es
ausfuehrt (auf einem PC mit 512 MB Hauptspeicher), passiert folgendes:

Ausgabe zur Standardausgabe stdout:

-----------------------------------------------------------
Verbunde02: Jetzt geht es los!
-----------------------------------------------------------
sizeof(VIEL_TEXT): 120000000
-----------------------------------------------------------
i1: 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20
-----------------------------------------------------------
i2: 1 2 3 4 5 6 7 8 9 10
---------------------------------------------------------------------------
Ausgabe zur Standardfehlerausgabe stderr:

      4 [main] a 2816 handle_exceptions: Exception: STATUS_ACCESS_VIOLATION
  39396 [main] a 2816 open_stackdumpfile: Dumping stack trace to a.exe.stackdump
------------------------------------------------------------------------ */
