// Datei Zeiger03.c
/* ------------------------------------------------------------------------
In Java unterscheidet man primitive Typen und Referenztypen.
Eine primitive Variable hat einen primitiven Wert (in der Bojendarstellung:
ein Rechteck), eine Referenzvariable hat eine Referenz als Wert (in der
Bojendarstellung: ein Sechseck).

In C gibt es sozusagen jeden Typ in zwei Versionen: Als primitiven Typ
und als Referenztyp (in C-Terminologie: Als "normalen" Typ und als
Zeigertyp). Zum "normalen" Typ int gibt es den Zeigertyp int * (Zeiger auf
int), zum "normalen" Typ float gibt es den Zeigertyp float * (Zeiger auf
float), zu einem "normalen" vom Programmierer vereinbarten Varbundtyp
Ding gibt es den Zeigertyp Ding * (Zeiger auf Ding) etc.
------------------------------------------------------------------------ */
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
#include <stdlib.h>  // atof, atoi, rand, malloc, calloc, free, exit,
                     // bsearch, qsort, abs, div, ...
// ------------------------------------------------------------------------
typedef struct {     // Dieser Verbundtyp heisst Ding
   short  us;        // 2 Bytes
   char   uc;        // 1 Byte
   double dd;        // 8 Bytes
} Ding;              // 2 + 1 + 8 gleich 16 Bytes ??
// ------------------------------------------------------------------------
// Eine Art "Konstruktor fuer Objekte des Typs Ding". Erzeugt eine
// neue Variable des Typs Ding, initialisiert sie und liefert ihre
// Adresse:
Ding * newDing(short us, char uc, double dd){
   Ding * zap = (Ding *) malloc(sizeof(Ding));
   (*zap).us = us;
   (*zap).uc = uc;
   (*zap).dd = dd;
   return zap;
} // newDing
// ------------------------------------------------------------------------
void gibAusDing1(Ding d) {
   // Gibt die Adressen und Werte der Komponenten des Verbundes d aus:
   printf("D sizeof(d):    %d Bytes\n", sizeof(d));
   printf("E    &d:        %p\n", &d);
   printf("F    &d.us:     %p, d.us:     %4d\n",   &d.us, d.us);
   printf("G    &d.uc:     %p, d.uc:     %4d\n",   &d.uc, d.uc);
   printf("H    &d.dd:     %p, d.dd:     %4.1f\n", &d.dd, d.dd);
} // gibAusDing1
// ------------------------------------------------------------------------
void gibAusDing2(Ding * zd) {
   // Gibt die Adressen und Werte der Komponenten des Verbundes *zd aus:
   printf("I sizeof(zd):   %d Bytes\n", sizeof(zd));
   printf("J    zd:        %p\n", zd);
   printf("K    &(*zd).us: %p, (*zd).us: %4d\n",   &(*zd).us, (*zd).us);
   printf("L    &(*zd).uc: %p, (*zd).uc: %4d\n",   &(*zd).uc, (*zd).uc);
   printf("M    &(*zd).dd: %p, (*zd).dd: %4.1f\n", &(*zd).dd, (*zd).dd);
} // gibAusDing1
// ------------------------------------------------------------------------
int main() {
   Ding    d01 =        {1, 55, 75.8};
   Ding * pd02 = newDing(2, 23, 68.5);

   printf("Zeiger03: Jetzt geht es los!\n");
   printf("----------------------------------------\n");
   printf("A sizeof(Ding): %d Bytes\n", sizeof(Ding));
   printf("B    &d01:      %p\n", &d01);
   printf("C    pd02:      %p\n", pd02);
   printf("----------------------------------------\n");
   gibAusDing1(  d01);
   printf("----------------------------------------\n");
   gibAusDing1(*pd02);
   printf("----------------------------------------\n");
   gibAusDing2( &d01);
   printf("----------------------------------------\n");
   gibAusDing2( pd02);
   printf("----------------------------------------\n");
   return 0;
} // main
/* ------------------------------------------------------------------------
Ausgabe des Programms Zeiger03:

Zeiger03: Jetzt geht es los!
----------------------------------------
A sizeof(Ding): 16 Bytes
B    &d01:      0012FF7C
C    pd02:      00842C50
----------------------------------------
D sizeof(d):    16 Bytes
E    &d:        0012FF60
F    &d.us:     0012FF60, d.us:        1
G    &d.uc:     0012FF62, d.uc:       55
H    &d.dd:     0012FF68, d.dd:     75.8
----------------------------------------
D sizeof(d):    16 Bytes
E    &d:        0012FF60
F    &d.us:     0012FF60, d.us:        2
G    &d.uc:     0012FF62, d.uc:       23
H    &d.dd:     0012FF68, d.dd:     68.5
----------------------------------------
I sizeof(zd):   4 Bytes
J    zd:        0012FF7C
K    &(*zd).us: 0012FF7C, (*zd).us:    1
L    &(*zd).uc: 0012FF7E, (*zd).uc:   55
M    &(*zd).dd: 0012FF84, (*zd).dd: 75.8
----------------------------------------
I sizeof(zd):   4 Bytes
J    zd:        00842C50
K    &(*zd).us: 00842C50, (*zd).us:    2
L    &(*zd).uc: 00842C52, (*zd).uc:   23
M    &(*zd).dd: 00842C58, (*zd).dd: 68.5
----------------------------------------
------------------------------------------------------------------------ */
