// Datei Zeiger05.c
#include <stdio.h>   // E/A in Bytestroeme, printf, sprintf, ...
/* ------------------------------------------------------------------------
Zu jedem "normalen" Typ T gibt es 4 Adresstypen, deren Werte und Variable
"mit mehr oder weniger Schreibberechtigungen" verbunden sind:
      T *      :   Veraenderbare Adresse einer   veraenderbaren T-Variablen
const T *      :   Veraenderbare Adresse einer unveraenderbaren T-Variablen
      T * const: Unveraenderbare Adresse einer   veraenderbaren T-Variablen
const T * const: Unveraenderbare Adresse einer unveraenderbaren T-Variablen

Dieses Programm zeigt die Unterschiede zwischen diesen 3 Typen.
------------------------------------------------------------------------ */
void machWas(      int *       a1, // Variable  Adr. einer int-Variablen
             const int *       a2, // Variable  Adr. einer int Konstanten
                   int * const a3, // Konstante Adr. einer int-Variablen
             const int * const a4, // Konstante Adr. einer int-Konstanten
                   int *       b)  // Variable  Adr. einer int-Variablen
   {
   // Die Parameter a1 bis a4 und b dieser Funktion sind Adressen von
   // int-Variablen. Diese Adressen sind "mit mehr oder weniger Schreib-
   // berechtigungen" verbunden. In dieser Funktion wird versucht,
   // die int-Variablen   *a1, *a2, ... etc. und
   // die Adressvariablen  a1,  a2, ... etc. zu veraendern.
   // Die Werte der int-Variablen und die Zielwerte der Adressvariablen
   // werden vor und nach den Veraenderungen ausgegeben.

   int * b1;
   int * b2;
   int * b3;
   int * b4;

   // Versuch, die int-Variablen *a1, *a2, ... etc. zu veraendern:
   (*a1)++; //       Erlaubt (die Variable *a1 ist   veraenderbar)
// (*a2)++; // Nicht erlaubt (die Variable *a2 ist unveraenderbar)
   (*a3)++; //       Erlaubt (die Variable *a3 ist   veraenderbar)
// (*a4)++; // Nicht erlaubt (die Variable *a4 ist unveraenderbar)

   // Versuch, die Adressvariablen a1, a2, ... etc. zu veraendern:
   a1 = b;  //       Erlaubt (die Variable  a1 ist   veraenderbar)
   a2 = b;  //       Erlaubt (die Variable  a2 ist   veraenderbar)
// a3 = b;  // Nicht erlaubt (die Variable  a3 ist unveraenderbar)
// a4 = b;  // Nicht erlaubt (die Variable  a4 ist unveraenderbar)

   printf("A *a1: %d, *a2: %d, *a3: %d, *a4: %d\n", *a1, *a2, *a3, *a4);

   // Die Adressen a1, a2, ... etc., die zum Teil nicht mit "voller
   // Schreibberechtigung verbunden sind", werden in Adressen
   // b1, b2, ... umgewandelt, die mit voller Schreibberechtigung
   // verbunden sind:
   b1 = (int *) a1;
   b2 = (int *) a2;
   b3 = (int *) a3;
   b4 = (int *) a4;

   (*b1)++;
   (*b2)++;
   (*b3)++;
   (*b4)++;

   b1 = b;
   b2 = b;
   b3 = b;
   b4 = b;

   printf("B *a1: %d, *a2: %d, *a3: %d, *a4: %d\n", *a1, *a2, *a3, *a4);
   printf("C *b1: %d, *b2: %d, *b3: %d, *b4: %d\n", *b1, *b2, *b3, *b4);
} // machWas
// ------------------------------------------------------------------------
void machNix(void) {
   // Diese Funktion "macht nix". Sie soll zeigen, dass man
   // anstelle von "const int" auch "int const" schreiben darf:
   const int *       a2;
   int const *       b2;
   const int * const a4;
   int const * const b4;

} // machNix
// ------------------------------------------------------------------------
int main() {
   int n = 17;
   int m = 65;



   printf("Zeiger05: Jetzt geht es los!\n");
   printf("------------------------------------\n");
   machWas(&n, &n, &n, &n, &m);
   printf("------------------------------------\n");
   printf("D n: %d, m: %d\n", n, m);
   printf("------------------------------------\n");

   return 0;
} // main
/* ------------------------------------------------------------------------
---------------------------------------------------------------------------
Ausgabe des Programms Zeiger05:

Zeiger05: Jetzt geht es los!
------------------------------------
A *a1: 65, *a2: 65, *a3: 19, *a4: 19
B *a1: 67, *a2: 67, *a3: 21, *a4: 21
C *b1: 67, *b2: 67, *b3: 67, *b4: 67
------------------------------------
D n: 21, m: 67
------------------------------------
------------------------------------------------------------------------ */
